@@ -10,14 +10,15 @@ from typing import TYPE_CHECKING, Any
 from uuid import uuid4
 
 import aiohttp
-import ujson
 from aiohttp.helpers import sentinel  # noqa:
 from apscheduler.jobstores.base import JobLookupError
 from dacite import from_dict
 from expiringdict import ExpiringDict
 from multidict import CIMultiDictProxy
 from packaging.version import Version, parse
+from yarl import URL
 
+from pylav.compat import json
 from pylav.constants.builtin_nodes import BUNDLED_NODES_IDS_HOST_MAPPING, PYLAV_NODES
 from pylav.constants.coordinates import REGION_TO_COUNTRY_COORDINATE_MAPPING
 from pylav.constants.node import GOOD_RESPONSE_RANGE, MAX_SUPPORTED_API_MAJOR_VERSION
@@ -120,7 +121,7 @@ class Node:
         self._version: Version | None = None
         self._api_version: int | None = None
         self._manager = manager
-        self._session = aiohttp.ClientSession(timeout=aiohttp.ClientTimeout(total=120), json_serialize=ujson.dumps)
+        self._session = aiohttp.ClientSession(timeout=aiohttp.ClientTimeout(total=120), json_serialize=json.dumps)
         self._temporary = temporary
         if not temporary:
             # noinspection PyProtectedMember
@@ -958,54 +959,56 @@ class Node:
             }
             return self.parse_loadtrack_response(data)
 
+    @property
+    def base_url(self) -> URL:
+        return URL(f"{self.connection_protocol}://{self.host}:{self.port}")
+
+    @property
+    def base_api_url(self) -> URL:
+        return self.base_url / f"v{self.api_version}"
+
     # ENDPOINTS
-    def get_endpoint_websocket(self) -> str:
-        return f"{self.socket_protocol}://{self.host}:{self.port}/v{self.api_version}/websocket"
+    def get_endpoint_websocket(self) -> URL:
+        return self.base_api_url / "websocket"
 
-    def get_endpoint_info(self) -> str:
-        return f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}/info"
+    def get_endpoint_info(self) -> URL:
+        return self.base_api_url / "info"
 
-    def get_endpoint_session_players(self) -> str:
-        return f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}/sessions/{self.session_id}/players"
+    def get_endpoint_session_players(self) -> URL:
+        return self.get_endpoint_session() / "players"
 
-    def get_endpoint_session_player_by_guild_id(self, guild_id: int) -> str:
-        return (
-            f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}"
-            f"/sessions/{self.session_id}/players/{guild_id}"
-        )
+    def get_endpoint_session_player_by_guild_id(self, guild_id: int) -> URL:
+        return self.get_endpoint_session_players() / f"{guild_id}"
 
-    def get_endpoint_session(self) -> str:
-        return f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}/sessions/{self.session_id}"
+    def get_endpoint_session(self) -> URL:
+        return self.base_api_url / "sessions" / self.session_id
 
-    def get_endpoint_loadtracks(self) -> str:
-        return f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}/loadtracks"
+    def get_endpoint_loadtracks(self) -> URL:
+        return self.base_api_url / "loadtracks"
 
-    def get_endpoint_decodetrack(self) -> str:
-        return f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}/decodetrack"
+    def get_endpoint_decodetrack(self) -> URL:
+        return self.base_api_url / "decodetrack"
 
-    def get_endpoint_decodetracks(self) -> str:
-        return f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}/decodetracks"
+    def get_endpoint_decodetracks(self) -> URL:
+        return self.base_api_url / "decodetracks"
 
-    def get_endpoint_stats(self) -> str:
-        return f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}/stats"
+    def get_endpoint_stats(self) -> URL:
+        return self.base_api_url / "stats"
 
-    def get_endpoint_routeplanner_status(self) -> str:
-        return f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}/routeplanner/status"
+    def get_endpoint_routeplanner_status(self) -> URL:
+        return self.base_api_url / "routeplanner" / "status"
 
-    def get_endpoint_routeplanner_free_address(self) -> str:
-        return f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}routeplanner/free/address"
+    def get_endpoint_routeplanner_free_address(self) -> URL:
+        return self.base_api_url / "routeplanner" / "free" / "address"
 
-    def get_endpoint_routeplanner_free_all(self) -> str:
-        return f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}/routeplanner/free/all"
+    def get_endpoint_routeplanner_free_all(self) -> URL:
+        return self.base_api_url / "routeplanner" / "free" / "all"
 
-    def get_endpoint_session_player_sponsorblock_categories(self, guild_id: int) -> str:
-        return (
-            f"{self.connection_protocol}://{self.host}:{self.port}/v{self.api_version}"
-            f"/sessions/{self.session_id}/players/{guild_id}/sponsorblock/categories"
-        )
+    def get_endpoint_session_player_sponsorblock_categories(self, guild_id: int) -> URL:
+        return self.get_endpoint_session_player_by_guild_id(guild_id) / "sponsorblock" / "categories"
 
-    def get_endpoint_version(self) -> str:
-        return f"{self.connection_protocol}://{self.host}:{self.port}/version"
+    def get_endpoint_version(self) -> URL:
+        return self.base_url / "version"
 
     # REST API - Direct calls
     async def fetch_session_players(self) -> list[rest_api.LavalinkPlayer] | HTTPException:
@@ -1027,10 +1030,8 @@ class Node:
             params={"trace": "true" if self.trace else "false"},
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
-                return [
-                    from_dict(data_class=rest_api.LavalinkPlayer, data=t) for t in await res.json(loads=ujson.loads)
-                ]
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+                return [from_dict(data_class=rest_api.LavalinkPlayer, data=t) for t in await res.json(loads=json.loads)]
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace("Failed to get session players: %d %s", failure.status, failure.message)
@@ -1047,8 +1048,8 @@ class Node:
             params={"trace": "true" if self.trace else "false"},
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
-                return from_dict(data_class=rest_api.LavalinkPlayer, data=await res.json(loads=ujson.loads))
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+                return from_dict(data_class=rest_api.LavalinkPlayer, data=await res.json(loads=json.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace("Failed to get session player: %d %s", failure.status, failure.message)
@@ -1068,8 +1069,8 @@ class Node:
             json=payload,
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
-                return from_dict(data_class=rest_api.LavalinkPlayer, data=await res.json(loads=ujson.loads))
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+                return from_dict(data_class=rest_api.LavalinkPlayer, data=await res.json(loads=json.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace("Failed to patch session player: %d %s", failure.status, failure.message)
@@ -1087,7 +1088,7 @@ class Node:
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE or res.status in [404]:
                 return
-            response = await res.json(loads=ujson.loads)
+            response = await res.json(loads=json.loads)
             failure = from_dict(data_class=LavalinkError, data=response)
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
@@ -1104,8 +1105,8 @@ class Node:
             },
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
-                return await res.json(loads=ujson.loads)
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+                return await res.json(loads=json.loads)
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace(
@@ -1127,7 +1128,7 @@ class Node:
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
                 return
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace(
@@ -1146,7 +1147,7 @@ class Node:
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
                 return
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace(
@@ -1167,7 +1168,7 @@ class Node:
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
                 return
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace("Failed to delete session player: %d %s", failure.status, failure.message)
@@ -1187,10 +1188,10 @@ class Node:
             params={"identifier": query.query_identifier, "trace": "true" if self.trace else "false"},
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
-                result = await res.json(loads=ujson.loads)
+                result = await res.json(loads=json.loads)
                 asyncio.create_task(self.node_manager.client.query_cache_manager.add_query(query, result))
                 return self.parse_loadtrack_response(result)
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace("Failed to load track: %d %s", failure.status, failure.message)
@@ -1210,8 +1211,8 @@ class Node:
             timeout=timeout,
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
-                return from_dict(data_class=Track, data=await res.json(loads=ujson.loads))
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+                return from_dict(data_class=Track, data=await res.json(loads=json.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace("Failed to decode track: %d %s", failure.status, failure.message)
@@ -1233,8 +1234,8 @@ class Node:
             params={"trace": "true" if self.trace else "false"},
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
-                return [from_dict(data_class=Track, data=t) for t in await res.json(loads=ujson.loads)]
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+                return [from_dict(data_class=Track, data=t) for t in await res.json(loads=json.loads)]
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace("Failed to decode tracks: %d %s", failure.status, failure.message)
@@ -1253,8 +1254,8 @@ class Node:
             params={"trace": "true" if self.trace else "false"},
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
-                return from_dict(data_class=rest_api.LavalinkInfo, data=await res.json(loads=ujson.loads))
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+                return from_dict(data_class=rest_api.LavalinkInfo, data=await res.json(loads=json.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 if raise_on_error:
                     raise UnauthorizedException(failure)
@@ -1280,8 +1281,8 @@ class Node:
             params={"trace": "true" if self.trace else "false"},
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
-                return from_dict(data_class=websocket_responses.Stats, data=await res.json(loads=ujson.loads))
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+                return from_dict(data_class=websocket_responses.Stats, data=await res.json(loads=json.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 if raise_on_error:
                     raise UnauthorizedException(failure)
@@ -1306,7 +1307,7 @@ class Node:
                 text = await res.text()
                 version_from_header = self._process_version_from_headers(res.headers)
                 return parse(text) if SEMANTIC_VERSIONING.match(text) else version_from_header
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 if raise_on_error:
                     raise UnauthorizedException(failure)
@@ -1327,11 +1328,11 @@ class Node:
             params={"trace": "true" if self.trace else "false"},
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
-                data = await res.json(loads=ujson.loads)
+                data = await res.json(loads=json.loads)
                 data["type"] = data["class"]
                 del data["class"]
                 return from_dict(data_class=RoutePlannerStart, data=data)
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace("Failed to get routeplanner status: %d %s", failure.status, failure.message)
@@ -1350,7 +1351,7 @@ class Node:
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
                 return
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace("Failed to free routeplanner address: %d %s", failure.status, failure.message)
@@ -1368,7 +1369,7 @@ class Node:
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
                 return
-            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=ujson.loads))
+            failure = from_dict(data_class=LavalinkError, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException(failure)
             self._logger.trace("Failed to free all routeplanner addresses: %d %s", failure.status, failure.message)
@@ -1395,7 +1396,7 @@ class Node:
             params={"trace": "true" if self.trace else "false"},
         ) as res:
             if res.status in GOOD_RESPONSE_RANGE:
-                return from_dict(data_class=rest_api.LavalinkPlayer, data=await res.json(loads=ujson.loads))
+                return from_dict(data_class=rest_api.LavalinkPlayer, data=await res.json(loads=json.loads))
             if res.status in [401, 403]:
                 raise UnauthorizedException
         raise ValueError(f"Server returned an unexpected return code: {res.status}")
